package DSRG.MW;

import java.util.Vector;
import java.util.Enumeration;

/** this class holds a number of tuples. It's used for holding
 * relation extents.
 * @author koeller (08/01/2000) */

public class Table {


      
    /**
     * Debug flag
     */
    public static final boolean DEBUG = false;
    

    /** a number of tuples in this extent */
    private Vector tuples;

    /** the schema of the Table. A vector of Strings
        (attribute names) */
    private Vector attributeNames;

    /** the sizes of attribute in the attributes of the
        Table. A vector of Integer (lengths) */
    private Vector attributeSizes=null;


    /** the virtual name of this relation (for query breakdown) */
    private String relName=null;

    /** construction an empty relation extent with a pointer to an MKB
     *   relation object */
    /*
    public Table(Relation myPeer) {
        peerRelation = myPeer;
        }*/


    /** a new constructor without argument... trying to get rid of the
        peer object */

    public Table() {
        tuples  = new Vector();
    }

    /** constructs a relation extent object with a default size. This
     *   is good for efficiency (if the size of the relation to be
     *   stored is known) */
    
    /*    public Table(Relation myPeer, int capacity) {
        peerRelation = myPeer;
        tuples = new Vector(capacity);
        }*/

    /** constructs a relation extent object with a default size. This
     *   is good for efficiency (if the size of the relation to be
     *   stored is known) */
    
    public Table(int capacity) {
        //        peerRelation = null;
        tuples = new Vector(capacity);
    }

    /** a copy constructor used to copy the extent of this table. This
        is tricky since the values of the Tuples have to be copied
        properly. */

    public Table(Table oldTable) {
        if (DEBUG) System.err.println("Copy constructor of MW.Table: Copying table "+oldTable.getRelName()+".");
        relName = new String(oldTable.getRelName());
        tuples = new Vector(oldTable.getTuples().size());
        for (Enumeration enum=((Vector)oldTable.getTuples()).elements();
             enum.hasMoreElements();) {
            tuples.addElement(((Tuple)enum.nextElement()).clone());
        }
        attributeNames = (Vector)oldTable.getAttributeNames().clone();
        if (attributeSizes!=null)
            attributeSizes = (Vector)oldTable.getAttributeSizes().clone();
    }        

    /** return this Table's Vector of tuples (the reference,
        not a clone) */

    public Vector getTuples () {
        return tuples;
    }

    /** returns the n-th column (starting count with 0) as a vector of
        strings */
    public Vector getColumn(int index) {
        Vector result = new Vector();
        Tuple current;
        for (Enumeration enum=tuples.elements();enum.hasMoreElements();) {
            current = (Tuple)enum.nextElement();
            result.addElement(current.getVector().elementAt(index));
        }
        return result;
    }


    /** returns the named column as a vector of
        strings */
    public Vector getColumn(String attName) {
        int index=-1;
        for (int i = 0 ; i < attributeNames.size(); i++)
            if (((String)attributeNames.elementAt(i)).equalsIgnoreCase(attName))
                index = i;
        if (index<=-1) return null;
        else return getColumn(index);
    }

    /** gets the tuples at index i
     * @param i the index of the tuple to be retrieved
     * @return null if the index is invalid */
    public Tuple getTuple(int i) {
        if (tuples == null) return null;
        if (i>=tuples.size()) return null;
        return (Tuple)tuples.elementAt(i);
    }

    /** returns the number of tuples in this relation.
     * @return -1 if there are no tuples
     */
    public int getSize() {
        if (tuples==null) return -1;
        return tuples.size();
    }

    /** return this Table's Relation reference */
    // private Relation getRelationx() {
    //   return peerRelation;
    // }

    /** adds a tuple at the end of this vector. If Vector is empty,
     *   creates a new vector with 10 original entries. Bag
     *   semantics. */
    public void addTuple (Tuple t) {
        if (tuples==null) tuples = new Vector (10);
        tuples.addElement(t);
    }

    /** searches for a certain tuple with the tuple's equals method
     *   and removes it from the extent. This is done
     *   <code>multiplicity</code> times.
     * @return the actual number of removed tuples. If this is less
     * than <code>multiplicity</code> of 0, then something is wrong */
    public int removeTuple (Tuple t, int multiplicity) {
        
        int removeCounter = 0;
        for (int j = 0 ; j < multiplicity ; j++) {
            for (int i = 0 ; i < tuples.size() ; i++ ) {
                if (i >= tuples.size()) break; 
                // since the vector may have become shorter in this loop;
                if (((Tuple)tuples.elementAt(i)).equals(t)) {
                    tuples.removeElementAt(i);
                    removeCounter++; // one tuple removed
                }
            }
        }
        return removeCounter;
    }
    
    /** "joins" this table with another. Essentially, expects a table
     * as input and stores a value from the last column in that table
     * into each value in the column of the same name of "this"
     * table. The other columns in the table are used to match
     * values. It is assumed that both table are sorted by the same
     * key (namely the first n-1 columns of the parameter table). This
     * function is needed for the UNFOLD-operator.  */

    public boolean setColumn(Table currentColumn) {
        //        if (currentColumn.getAttributeNames().size()!=1)
        //    return false;
        Vector atts=currentColumn.getAttributeNames();
        int columns= atts.size();
        int colIndex = attributeNames.indexOf 
            (atts.elementAt(columns-1));
        if (colIndex <0) 
            return false;
        //if (getSize() != currentColumn.getSize())
        //    return false;
        
        // now "join". The this-table must have more or equally many
        // row than the that-table since its rows are the union of all
        // that-tables. So whenever key values don't match, we skip a
        // row in the this-table - the that-table never skips.
        Tuple currentThis, currentThat;
        Vector thisVector,thatVector; 
        Enumeration
            thatE=currentColumn.getTuples().elements();
        Enumeration thisE=tuples.elements();
        boolean matches;
        currentThat = (Tuple)thatE.nextElement();
        while(thisE.hasMoreElements()) {
            currentThis = (Tuple)thisE.nextElement();
            thisVector = currentThis.getVector();
            thatVector = currentThat.getVector();
            matches = true;
            // for all key columns
            for (int i = 0 ; i < atts.size()-1 && matches; i++) {
                // compare values
                if (!thisVector.elementAt(i).equals(thatVector.elementAt(i)))
                    matches=false; // and break
            }
            if (matches) {
                thisVector.setElementAt(thatVector.elementAt(columns-1),colIndex);
                if (thatE.hasMoreElements())
                    currentThat = (Tuple)thatE.nextElement();
            }
        }
        
        return true;
    }


    /** string representation (calls toString for all tuples, one line each).
     * @return null if no tuples in the relation */
    public String toString() {
        String s = "TABLE\nName: "+relName+"\n";
        for (Enumeration e=attributeNames.elements();e.hasMoreElements();)
            s+=((String)e.nextElement())+"|";
        s+="\n============================================================\n";
        if (tuples == null) return null;
        Enumeration e = tuples.elements();
        while (e.hasMoreElements()) {
            s += ((Tuple)e.nextElement()).toString()+"\n";
        }
        s+="END OF TABLE\n";
        return s;
    }


    public String getRelName() {
        return relName;
    }
    
    public void setRelName(String name) {
        relName = name;
    }

    public Vector getAttributeNames() {
        return attributeNames;
    }

    public void setAttributeNames(Vector attNames) {
        attributeNames = attNames;
    }

    public Vector getAttributeSizes() {
        return attributeSizes;
    }

    public void setAttributeSizes(Vector attSizes) {
        attributeSizes = attSizes;
    }
}





