package DSRG.MW;

import java.util.Vector;
import java.util.Enumeration;

/** this class holds a tuple in a relation object. This is used for
 * storing (cacheing) relational extents. 
 * @author koeller (08/01/2000) */


public class Tuple { 
    
    /** the values in the tuple. The types of the elements are
     *   heterogeneous, but have to agree with the types in the
     *   "attribute" objects referenced in the attributes-vector in
     *   this tuple's Table-object. */
    Vector values; 

    /** The extent to which this tuple belongs. Tuples belong to
     *   exactly one Table instance */
    Table parent;


    /** constructs a tuple with a certain Table instance as
     *   parent and sets its value from a vector (not cloned). */
    public Tuple (Table myParent, Vector myValues) {
        parent = myParent;
        values = myValues;
    }

    /** returns an enumeration on this tuple. The elements are of type
     *  "Object" */
    public Enumeration getEnumeration () {
        return values.elements();
    }

    /** returns the number of elements (attributes) in thi s
     * tuple. This should correspond to the number of attributes in
     * this tuple's relation extent's relation object, but this is not
     * enforced or checked... */
    public int getSize() {
        return values.size();
    }

    /** returns this tuple as a vector */
    public Vector getVector () {
        return (Vector)values;
    }
    
    /** sets this tuple's vector (=data). Does not clone the
     *   contents. */
    public void setVector(Vector myValues) {
        values = (Vector)myValues;
    }

    /** recursively calls the equals methods of this vector's objects */
    
    public boolean equals (Tuple otherTuple) {
        if (values.size() != otherTuple.getVector().size()) return false;
        Enumeration otherEnum = otherTuple.getEnumeration();
        Enumeration thisEnum = getEnumeration();
        while (otherEnum.hasMoreElements()) {
            if (!thisEnum.nextElement().equals(otherEnum.nextElement()))
                return false;
        }
        return true;
    }
            

    /** calls the toString() methods of this tuple's objects and
        concatenates them */
    public String toString () {
        String s= new String();
        for (int i=0; i<values.size(); i++) {
            if (values.elementAt(i)==null) 
                s += "@@NULL@@|" ;
            else
                s += values.elementAt(i).toString()+"|";
        }
        return s;
    }

    /** calls the toString() methods of this tuple's objects and
     *   concatenates them with commas in between and single quotes
     *   around the strings. This is used for storing this tuple in a
     *   JDBC database */
    public String toSQLString () {
        String s= new String();
        for (int i=0; i<values.size(); i++) {
            if (values.elementAt(i)==null) 
                s += "null," ;
            else
                s += "'" + values.elementAt(i).toString()+"',";
        }
        return s.substring(0,s.length()-1); // cut last comma
    }    

    /** Clones this tuple and assumes all values are strings. This is
     *  a pretty bad hack, but I couldn't see any other way... */
    public  synchronized Object clone() {
        Vector tempValues=new Vector(values.size());
        String currentElement;
        for (Enumeration enum = values.elements();enum.hasMoreElements();) {
            currentElement = (String)enum.nextElement();
            if (currentElement==null)
                tempValues.addElement(null);
            else
                tempValues.addElement(new String(currentElement));
        }
        Tuple tempTuple = new Tuple(parent,tempValues);
        return tempTuple;
    }
}





