import java.util.Vector;

import java.io.*;
import DSRG.MW.*;
import treeBuilder.*;
import algebraTree.*;
import schemaSQLParser.*;
import schemaSQLParser.syntaxtree.*;
import schemaSQLParser.visitor.*;

class Main {
    
    static Builder treeBuilder;
    static QENode algebraTreeRoot;
    static long middletime;

    public static void main(String args[]) throws ParseException {
        DBManager localDB = 
            new DBMOracle("LOCALIS","eve","eve",
                          "jdbc:oracle:thin:@shiba.wpi.edu:1521:ORCL");
        DBManager sourceDB =
            new DBMOracle("SourceIS","schemasql","schemasql",
                          "jdbc:oracle:thin:@wpi.wpi.edu:1521:CS");
        
        treeBuilder = new Builder(sourceDB,localDB);

        SchemaSql p = null;
        if ( args.length < 1 ) {
            System.out.println("Reading from stdin");
            p = new SchemaSql(System.in);
        }
        else {
            try {
                p = new SchemaSql(new DataInputStream(new FileInputStream(args[0])));
            }
            catch (FileNotFoundException e) {
                System.out.println("File " + args[0]+ " not found. Reading from stdin");
                p = new SchemaSql(System.in);
            }
        }
        if ( args.length > 0 ) {
            System.out.println(args[0]);
        }
        p.disable_tracing();
        Node parseTreeRoot = p.CompilationUnit();
        System.err.println("SchemaSQL parsed successfully.");
        SchemaSQLVisitor vis = new SchemaSQLVisitor(treeBuilder);
        parseTreeRoot.accept(vis);

        System.err.println("Query tree successfully evaluated.");
        algebraTreeRoot = treeBuilder.createAlgebraTree();
        boolean nocompute = false;
        if ( args.length == 2 ) {
            if (args[1].equals("-parseonly"))
                parseTreeRoot.accept(new TreePrinter());
            else if (args[1].equals("-nocompute"))
                nocompute = true;
        }

        /*
        System.out.println("ALGEBRA-TREE:");
        algebraTreeRoot.printTree("");
        System.out.println("END OF ALGEBRA-TREE.");
        */

        long starttime = System.currentTimeMillis();
        System.out.println("Computing view");
        if (!nocompute) algebraTreeRoot.computeResult();
        middletime  = System.currentTimeMillis();


        //System.out.println("Use: \n"+"select * from "+algebraTreeRoot.getRelName()+"\n to query.");
        //Table result = localDB.executeViewQuery("select * from "+algebraTreeRoot.getRelName());
        //System.out.println(result.toString());
        System.out.println("Executing updates");

        //DEL-REL for airline2.sql 
        String delRelName = "AIRLINE_AF";
        //        algebraTreeRoot.executeUpdate(new SchemaUpdate(delRelName,SchemaUpdate.DELETE));


        /* 111 DELETE-TUPLEs from AIRLINE_KL in the 111/222/333/444-scheme */
        /* 222 DELETE-TUPLEs from AIRLINE_AF in the 111/222/333/444-scheme */
        // we use the "recomputation" value from the old experiment
        ///... there was some stuff here

        /* 10 ADD-TUPLEs, then 10 deletes, then 10 more adds for
           airline2.sql (experiments) */
        /*
          insertTupleAirline2("AIRLINE_KL","Amsterdam","1000","0601");
          insertTupleAirline2("AIRLINE_KL","Liege","1001","0602");
          insertTupleAirline2("AIRLINE_KL","Utrecht","1002","0603");
          insertTupleAirline2("AIRLINE_KL","Den Haag","1003","0604");
          insertTupleAirline2("AIRLINE_KL","Bruxelles","1004","0605");
          insertTupleAirline2("AIRLINE_KL","Rotterdam","1015","0606");
          insertTupleAirline2("AIRLINE_KL","Schengen","1016","0607");
          insertTupleAirline2("AIRLINE_KL","Eupen","1017","0608");
          insertTupleAirline2("AIRLINE_KL","Malmedy","1018","0609");
          insertTupleAirline2("AIRLINE_KL","Maastricht","1019","0610");

          deleteTupleAirline2("AIRLINE_AF","Saint_Marys","1094","0303");
        deleteTupleAirline2("AIRLINE_AF","Kitchener","1017","0396");
        deleteTupleAirline2("AIRLINE_AF","Murud","1128","0306");
        deleteTupleAirline2("AIRLINE_AF","Kalimpang","1101","0581");
        deleteTupleAirline2("AIRLINE_AF","Magdeburg","1041","0517");
        deleteTupleAirline2("AIRLINE_AF","Tianjin","1114","0606");
        deleteTupleAirline2("AIRLINE_AF","Zhenjiang","1069","0522");
        deleteTupleAirline2("AIRLINE_AF","Anshun","1163","0370");
        deleteTupleAirline2("AIRLINE_AF","Moose_Factory","1035","0456");
        deleteTupleAirline2("AIRLINE_AF","Guangzhou","1170","0479");        

        deleteTupleAirline2("AIRLINE_KL","Amsterdam","1000","0601");
        deleteTupleAirline2("AIRLINE_KL","Liege","1001","0602");
        deleteTupleAirline2("AIRLINE_KL","Utrecht","1002","0603");
        deleteTupleAirline2("AIRLINE_KL","Den Haag","1003","0604");
        deleteTupleAirline2("AIRLINE_KL","Bruxelles","1004","0605");
        deleteTupleAirline2("AIRLINE_KL","Rotterdam","1015","0606");
        deleteTupleAirline2("AIRLINE_KL","Schengen","1016","0607");
        deleteTupleAirline2("AIRLINE_KL","Eupen","1017","0608");
        deleteTupleAirline2("AIRLINE_KL","Malmedy","1018","0609");
        deleteTupleAirline2("AIRLINE_KL","Maastricht","1019","0610");
        deleteTupleAirline2("AIRLINE_KL","DUMMY","0000","0000");
              
        deleteTupleAirline2("AIRLINE_LH","Gongxian","1006","0532");
        deleteTupleAirline2("AIRLINE_LH","Mengxian","1133","0698");
        deleteTupleAirline2("AIRLINE_LH","Cholet","1022","0417");
        deleteTupleAirline2("AIRLINE_LH","Wolfsburg","0928","0510");
        deleteTupleAirline2("AIRLINE_LH","Lucan","0946","0613");
        deleteTupleAirline2("AIRLINE_LH","Lorient","1159","0423");
        deleteTupleAirline2("AIRLINE_LH","Jinghai","1159","0641");
        deleteTupleAirline2("AIRLINE_LH","Gonggar","0920","0465");
        deleteTupleAirline2("AIRLINE_LH","St_John_s","1093","0608");
        deleteTupleAirline2("AIRLINE_LH","Moonbeam","0945","0636");         
        */     
        /* 2 ADD-TUPLEs for airline.sql (paper) */
        /*
        {   
            Vector tmpData = new Vector(2);
            Vector tmpAttNames = new Vector(2);
            Vector tmpAttributeSizes = new Vector(2);
            
            tmpAttNames.addElement("DESTINATION");
            tmpAttNames.addElement("BUSINESS");
            tmpAttNames.addElement("ECONOMY");
            tmpData.addElement("Berlin");
            tmpData.addElement("1400");
            tmpData.addElement("0610");
            
            Table tmpTable = new Table(1);
            tmpTable.setRelName("DUMMY");
            tmpTable.setAttributeNames(tmpAttNames);
            Tuple tmpTuple = new Tuple(tmpTable,tmpData);
            tmpTable.addTuple(tmpTuple);
            
            DataUpdate update = new DataUpdate("AIRLINE_LH",null,tmpTable);
            algebraTreeRoot.executeUpdate(update);
        }
        */
        /*
        {   
            Vector tmpData = new Vector(2);
            Vector tmpAttNames = new Vector(2);
            Vector tmpAttributeSizes = new Vector(2);
            
            tmpAttNames.addElement("DESTINATION");
            tmpAttNames.addElement("BUSINESS");
            tmpAttNames.addElement("ECONOMY");
            tmpData.addElement("Berlin");
            tmpData.addElement("1300");
            tmpData.addElement("0660");
            
            Table tmpTable = new Table(1);
            tmpTable.setRelName("DUMMY");
            tmpTable.setAttributeNames(tmpAttNames);
            Tuple tmpTuple = new Tuple(tmpTable,tmpData);
            tmpTable.addTuple(tmpTuple);
            
            DataUpdate update = new DataUpdate("AIRLINE_BA",null,tmpTable);
            algebraTreeRoot.executeUpdate(update);
        }
        */
        //============================================================
        //============================================================
        //============================================================
        // for UNITE/FOLD operators
        //============================================================
        //============================================================
        //============================================================
        //============================================================



        /* DELETE-RELATION for salinfo_ctoa.sql */

        /*        String delRelName = "UNI_PHYS";
        algebraTreeRoot.executeUpdate(new SchemaUpdate(delRelName,SchemaUpdate.DELETE));
        */

        /* ADD-ATTRIBUTE for salinfo_ctoa.sql */
        /*
        algebraTreeRoot.executeUpdate(new SchemaUpdate("UNI_PHYS","Stuff",SchemaUpdate.ADD));
        algebraTreeRoot.executeUpdate(new SchemaUpdate("UNI_MATH","Stuff",SchemaUpdate.ADD));
        algebraTreeRoot.executeUpdate(new SchemaUpdate("UNI_CS","Stuff",SchemaUpdate.ADD));
        */
        /* ADD-ATTRIBUTE */
        /*
        algebraTreeRoot.executeUpdate(new SchemaUpdate("salinfob","Stuff",SchemaUpdate.ADD));
        */
        
        /* RENAME-ATTRIBUTE */
        /*
        algebraTreeRoot.executeUpdate(new SchemaUpdate("salinfob","CS","Inf",SchemaUpdate.RENAME));
        */

        /* DELETE-ATTRIBUTE */
        /*
        algebraTreeRoot.executeUpdate(new SchemaUpdate("salinfob","INF",SchemaUpdate.DELETE));
        */

        /* ADD-TUPLE */
        // adding a tuple to salinfob
        /*
     {   Vector tmpData = new Vector(2);
        Vector tmpAttNames = new Vector(2);
        Vector tmpAttributeSizes = new Vector(2);
        
        tmpAttNames.addElement("CATEGORY");
        tmpAttNames.addElement("CS");
        tmpAttNames.addElement("MATH");
        tmpData.addElement("Janitor");
        tmpData.addElement("11000");
        tmpData.addElement("10000");

        Table tmpTable = new Table(1);
        tmpTable.setRelName("DUMMY");
        tmpTable.setAttributeNames(tmpAttNames);
        Tuple tmpTuple = new Tuple(tmpTable,tmpData);
        tmpTable.addTuple(tmpTuple);
        
        DataUpdate update = new DataUpdate("salinfob",null,tmpTable);
        algebraTreeRoot.executeUpdate(update);
    }
        */
        /* DELETE-TUPLE */
        // deleting a tuple from salinfob
        /*
     {
         Vector tmpData = new Vector(2);
         Vector tmpAttNames = new Vector(2);
         Vector tmpAttributeSizes = new Vector(2);
         
         tmpAttNames.addElement("CATEGORY");
         tmpAttNames.addElement("CS");
         tmpAttNames.addElement("MATH");
         tmpData.addElement("Prof");
         tmpData.addElement("55000");
         tmpData.addElement("65000");
         
         Table tmpTable = new Table(1);
         tmpTable.setRelName("DUMMY");
         tmpTable.setAttributeNames(tmpAttNames);
         Tuple tmpTuple = new Tuple(tmpTable,tmpData);
         tmpTable.addTuple(tmpTuple);

         DataUpdate update = new DataUpdate("salinfob",tmpTable,null);
         algebraTreeRoot.executeUpdate(update);
     }
        */
        //============================================================
        //============================================================
        //============================================================
        // for UNFOLD/SPLIT operators
        //============================================================
        //============================================================
        //============================================================
        //============================================================
        
        /* ADD-ATTRIBUTE */
        //        algebraTreeRoot.executeUpdate(new SchemaUpdate("salinfoa","Schlumpf",SchemaUpdate.ADD));
        
        /* DELETE-ATTRIBUTE */
        //        algebraTreeRoot.executeUpdate(new SchemaUpdate("salinfoa","Category",SchemaUpdate.DELETE));

        /* ADD-TUPLE */
        // adding a tuple to salinfoa
        /*
     {   Vector tmpData = new Vector(2);
        Vector tmpAttNames = new Vector(2);
        Vector tmpAttributeSizes = new Vector(2);
        
        tmpAttNames.addElement("CATEGORY");
        tmpAttNames.addElement("DEPT");
        tmpAttNames.addElement("SALFLOOR");
        tmpData.addElement("Janitor");
        tmpData.addElement("CS");
        tmpData.addElement("10000");

        Table tmpTable = new Table(1);
        tmpTable.setRelName("DUMMY");
        tmpTable.setAttributeNames(tmpAttNames);
        Tuple tmpTuple = new Tuple(tmpTable,tmpData);
        tmpTable.addTuple(tmpTuple);
        
        DataUpdate update = new DataUpdate("salinfoa",null,tmpTable);
        algebraTreeRoot.executeUpdate(update);
    }
        */
        /* DELETE-TUPLE */
        // deleting a tuple from salinfoa
        /*
     {
         Vector tmpData = new Vector(2);
         Vector tmpAttNames = new Vector(2);
         Vector tmpAttributeSizes = new Vector(2);
         
         tmpAttNames.addElement("CATEGORY");
         tmpAttNames.addElement("DEPT");
         tmpAttNames.addElement("SALFLOOR");
         tmpData.addElement("Prof");
         tmpData.addElement("CS");
         tmpData.addElement("65000");
         
         Table tmpTable = new Table(1);
         tmpTable.setRelName("DUMMY");
         tmpTable.setAttributeNames(tmpAttNames);
         Tuple tmpTuple = new Tuple(tmpTable,tmpData);
         tmpTable.addTuple(tmpTuple);

         DataUpdate update = new DataUpdate("salinfoa",tmpTable,null);
         algebraTreeRoot.executeUpdate(update);
     }
        */
     /*
     {
         Vector tmpData = new Vector(2);
         Vector tmpAttNames = new Vector(2);
         Vector tmpAttributeSizes = new Vector(2);
         
         tmpAttNames.addElement("CATEGORY");
         tmpAttNames.addElement("DEPT");
         tmpAttNames.addElement("SALFLOOR");
         tmpData.addElement("Assoc Prof");
         tmpData.addElement("CS");
         tmpData.addElement("50000");
         
         Table tmpTable = new Table(1);
         tmpTable.setRelName("DUMMY");
         tmpTable.setAttributeNames(tmpAttNames);
         Tuple tmpTuple = new Tuple(tmpTable,tmpData);
         tmpTable.addTuple(tmpTuple);

         DataUpdate update = new DataUpdate("salinfoa",tmpTable,null);
         algebraTreeRoot.executeUpdate(update);
     }
     */
        /*
     {
         Vector tmpData = new Vector(2);
         Vector tmpAttNames = new Vector(2);
         Vector tmpAttributeSizes = new Vector(2);
         
         tmpAttNames.addElement("CATEGORY");
         tmpAttNames.addElement("DEPT");
         tmpAttNames.addElement("SALFLOOR");
         tmpData.addElement("Janitor");
         tmpData.addElement("Bio");
         tmpData.addElement("10000");
         
         Table tmpTable = new Table(1);
         tmpTable.setRelName("DUMMY");
         tmpTable.setAttributeNames(tmpAttNames);
         Tuple tmpTuple = new Tuple(tmpTable,tmpData);
         tmpTable.addTuple(tmpTuple);

         DataUpdate update = new DataUpdate("salinfoa",tmpTable,null);
         algebraTreeRoot.executeUpdate(update);
     }
        */
     /* UPDATE-TUPLE */
     // updating a tuple
        /*
     {
         Vector tmpData = new Vector(2);
         Vector tmpAttNames = new Vector(2);
         Vector tmpAttributeSizes = new Vector(2);
         
         tmpAttNames.addElement("CATEGORY");
         tmpAttNames.addElement("DEPT");
         tmpAttNames.addElement("SALFLOOR");
         tmpData.addElement("Janitor");
         tmpData.addElement("Bio");
         tmpData.addElement("10000");
         
         Table tmpTable = new Table(1);
         tmpTable.setRelName("DUMMY");
         tmpTable.setAttributeNames(tmpAttNames);
         Tuple tmpTuple = new Tuple(tmpTable,tmpData);
         tmpTable.addTuple(tmpTuple);

         tmpData = new Vector(2);
         tmpAttNames = new Vector(2);
         tmpAttributeSizes = new Vector(2);
         
         tmpAttNames.addElement("CATEGORY");
         tmpAttNames.addElement("DEPT");
         tmpAttNames.addElement("SALFLOOR");
         tmpData.addElement("Janitor");
         tmpData.addElement("Bio");
         tmpData.addElement("20000");
         
         Table tmpTable2 = new Table(1);
         tmpTable2.setRelName("DUMMY");
         tmpTable2.setAttributeNames(tmpAttNames);
         tmpTuple = new Tuple(tmpTable2,tmpData);
         tmpTable2.addTuple(tmpTuple);
         
         DataUpdate update = new DataUpdate("salinfoa",tmpTable,tmpTable2);
         algebraTreeRoot.executeUpdate(update);
     }
        */
     //============================================================
        //============================================================
        // END of UPDATES
        //============================================================
        //============================================================


        //============================================================
        //============================================================
        // this block has to reside after the end of the active updates.
        //============================================================
        //============================================================


        long middletime2  = System.currentTimeMillis();
        // execute the updates against the sourceDB here
        // for airline.sql
        // sourceDB.executeSQLUpdate("insert into AIRLINE_LH values ('Berlin','1400','0620')");
        // for del-tuple  in airline2.sql
        //int execResult = sourceDB.executeSQLUpdate("delete from "+delRelName);

        //if (execResult==0) System.out.println("WARNING: executing physical update did not succeed.");

        /* // for salinfo_ctoa.sql: 
           sourceDB.deleteRelation(delRelName,true);
        */

        //for del-rel airline2.sql 
        //        sourceDB.deleteRelation(delRelName,true);

        long middletime3  = System.currentTimeMillis();
        System.out.println("Recomputing result");
        //if (!nocompute) algebraTreeRoot.computeResult();
        

        long endtime = System.currentTimeMillis();
        System.out.println("View computation took "+(middletime-starttime)/1000.0+" s.");
        System.out.println("Update propagation took "+(middletime2-middletime)/1000.0+" s.");
        System.out.println("Recomputation took "+(endtime-middletime3)/1000.0+" s.");
        System.err.println("DONE");
        // return to previous state
        
        java.awt.Toolkit.getDefaultToolkit().beep();        
    }

    static void deleteTupleAirline2(String relationName, String destination, String business, String economy) {
                    Vector tmpData = new Vector(2);
            Vector tmpAttNames = new Vector(2);
            Vector tmpAttributeSizes = new Vector(2);
            
            tmpAttNames.addElement("DESTINATION");
            tmpAttNames.addElement("BUSINESS");
            tmpAttNames.addElement("ECONOMY");
            tmpData.addElement(destination);
            tmpData.addElement(business);
            tmpData.addElement(economy);
            
            Table tmpTable = new Table(1);
            tmpTable.setRelName("DUMMY");
            tmpTable.setAttributeNames(tmpAttNames);
            Tuple tmpTuple = new Tuple(tmpTable,tmpData);
            tmpTable.addTuple(tmpTuple);
            
            DataUpdate update = new DataUpdate(relationName,tmpTable,null);
            algebraTreeRoot.executeUpdate(update);
            System.out.println(System.currentTimeMillis()-middletime);
        
    }

    static void insertTupleAirline2(String relationName, String destination, String business, String economy) {
        Vector tmpData = new Vector(2);
        Vector tmpAttNames = new Vector(2);
        Vector tmpAttributeSizes = new Vector(2);
        
        tmpAttNames.addElement("DESTINATION");
        tmpAttNames.addElement("BUSINESS");
        tmpAttNames.addElement("ECONOMY");
        tmpData.addElement(destination);
        tmpData.addElement(business);
        tmpData.addElement(economy);
        
        Table tmpTable = new Table(1);
        tmpTable.setRelName("DUMMY");
        tmpTable.setAttributeNames(tmpAttNames);
        Tuple tmpTuple = new Tuple(tmpTable,tmpData);
        tmpTable.addTuple(tmpTuple);
        
        DataUpdate update = new DataUpdate(relationName,null,tmpTable);
        algebraTreeRoot.executeUpdate(update);
        System.out.println(System.currentTimeMillis()-middletime);
    }
    
}
        



