package algebraTree;
import java.util.*;

import DSRG.MW.*;




/** A data update. This is logical only - the physical update should
 *   not have anything to do with this class!  */

public class DataUpdate extends Update {
    /** if "before" is null, we have an add, if "after" is null, we
     * have a delete, otherwise, we have an update. Both "before" and
     * "after" must be Tables with exactly one Tuple each. As we can
     * not tell if a key is updated or not, we have no choice but to
     * treat an "update" as an add following a delete; but for the
     * incremental update policy that should not matter */
    protected Table before,after;
    
    public DataUpdate(String relationP, Table beforeP, Table afterP) {
        relation = relationP;
        if (beforeP==null && afterP==null) 
            Util.exit("empty DataUpdate. Before- and After-Table cannot both be null");
        before = beforeP;
        after=afterP;
    }
    
    /** copy constructor for update propagation */
    public DataUpdate(DataUpdate old) {
        relation = new String(old.relation);
        if (old.before==null)
            before = null;
        else
            before = new Table(old.before);
        if (old.after==null)
            after = null;
        else
            after = new Table(old.after);
    }

    public String toString() {
        return "DataUpdate -- "
            +": Rel: "
            +relation
            +" From: "
            +(before==null?"null":before.toString())
            +" To: "
            +(after==null?"null":after.toString());
    }

}
