package algebraTree;
import java.util.*;

import DSRG.MW.*;

public abstract class SchemaSQLOperator {

    /** these are DEFINEs for the Leafness status of an operator */
    public final static boolean LEAF =true;
    /** these are DEFINEs for the Leafness status of an operator */
    public final static boolean NOLEAF = false;

    /** for intermediate results */
    protected DBManager localDB;
    /** for intermediate results */
    protected boolean isLeaf;
    
    /** I forgot what this was for... :( */
    protected String localQuery;

    /** the name of the output table of this operator. Makes no sense
        for SPLIT */
    protected String outputTableName;

    /** operator stores its pivotData for incremental update
     * propagation. This is a table with all distinct values of the
     * base table (String) plus their multiplicity (number of
     * occurrences) as a String too (!). This table will be valid only
     * after the initial call to <CODE>computeResult()</CODE> and will
     * then be maintained incrementally; */
    Table pivotData =  null;

    /** currently, only one source DB is supported. Distributed
        sources can be simulated by having "select *
        from"-StandardSQL-nodes that essentially copy base data to the
        local DB */
    DBManager sourceDB;

    public SchemaSQLOperator(DBManager myLocalDB, 
                             boolean leafness) {
        isLeaf = leafness;
        localDB = myLocalDB;
        if (!isLeaf) sourceDB=localDB;
        // number generator is static
        outputTableName = Util.getTempTableName();
    }        

    /** in the finalize method we drop local tables. I'm not sure if
        this will work since the finalize method behaves somewhat
        weird. */
    protected void finalize() {
        System.out.println("dropping table "+outputTableName);
        dropLocalTable();
    }
    
    // only for debugging purposes
    public void setQuery(String s) {
        if (!Main.DEBUG) System.err.println("Production version should not use setQuery() in class SchemaSQLOperator");
        localQuery = new String(s);
    }

    /** this is needed to find out what table name this operator has
     *  used. Mainly needed for the root node. Must be public since
     *  it's used in Builder::flatRelName */
    public String getRelName() {
        return outputTableName;
    }
    
    /** sets the relation name of this operator. Is needed from the
        outside to set the name of the root-operator's relation so the
        wrapper can produce named tables. */
    /* package private */void setRelName(String s) {
        outputTableName = new String(s);
    }
    
    public boolean setSourceDB(DBManager setSourceDB) {
        if (!isLeaf) return false;
        else {
            sourceDB = setSourceDB;
            return true;
        }
    }


    /** Executes a query passed in a parameter against this node's
        database. Uses DBManager.executeViewQuery() since
        executeQuery() needs relation names in the SQL which we don't
        have. */
    protected boolean executeQuery(String SQL) {
        Table temp =sourceDB.executeViewQuery(SQL);
        temp.setRelName(outputTableName);
        return localDB.storeRelation(temp);
    }

    /** executes all queries returned by computeLocalQueries().  This
        is quite primitive and overridden by the Unfold and Split
        operators! */
    protected boolean executeQuery() {
        boolean result = true;
        for (Enumeration enum=computeLocalQueries().elements();
             enum.hasMoreElements() && result;)
            result = executeQuery((String)enum.nextElement());
        return result;
    }

    /** drops this operator's table from the DB. Table will not be dropped if
        algebraTree.Main.DEBUG is true and in all cases the drop will
        be silent (error messages suppressed) */
    protected boolean dropLocalTable() {
        Table table;
        // silent delete
        if (Main.DEBUG) 
            return true;
        else 
            return localDB.deleteRelation(outputTableName,true);
    }

    /** returns a vector of strings of queries that have to be
        executed for this operator */
    abstract protected Vector computeLocalQueries();

    /** This function takes a vector of updates and applies it to the
     outputRelation of this operator.  Probably needs to be
     overwritten for the Split-Class*/
    /* package private*/ void executeUpdates(Vector updates) {


        Update upd;
        DataUpdate updD; SchemaUpdate updS;
        boolean firstTime;
        String query = new String();
        for (Enumeration enum = updates.elements();
             enum.hasMoreElements();) {

            upd = (Update)enum.nextElement();
            if (upd instanceof DataUpdate) {
                updD = (DataUpdate)upd;

                if (updD.before == null) {//add-tuple
                    //insert-query
                    String names = new String();
                    String values = new String();
                    String tmpValue;
                    for (int i = 0; i<updD.after.getAttributeNames().size(); i++) {
                        if (i>=1) { names += ","; values += ","; }
                        names += updD.after.getAttributeNames().elementAt(i);
                        tmpValue = (String)((Tuple)updD.after.getTuples().elementAt(0)).getVector().elementAt(i);
                        if (tmpValue==null) values += "null";
                        else values += "'"+tmpValue+"'";
                                                
                    }
                    query = "insert into "
                        + outputTableName
                        + " ("
                        + names
                        + ") values("
                        + values 
                        + ")";
                    int result = localDB.executeSQLUpdate(query);
                }
                else { // delete- or update-tuple
                    // construct where-clause
                    String whereClause = new String();
                    String tmpValue;
                    for (int i = 0; i<updD.before.getAttributeNames().size(); i++) {
                        if (i>=1) whereClause += " and ";
                        whereClause += updD.before.getAttributeNames().elementAt(i);
                        tmpValue = (String)((Tuple)updD.before.getTuples().elementAt(0)).getVector().elementAt(i);
                        if (tmpValue==null) whereClause += " is null";
                        else whereClause += "='"+tmpValue+"'";
                    }
                    // now whereClause contains a where clause for all tuple values in the before-tuple
                    if (updD.after==null) {//delete-tuple
                        query = "delete from "
                            + outputTableName
                            + " where "
                            + whereClause;
                        int result = localDB.executeSQLUpdate(query);
                        if (result==0) {
                            /* // not sure if this is correct 100900
                            // if we have a standardsql-node, this
                            // simply means the tuple is projected
                            // out, otherwise it's an error:
                            //                            if (!(this instanceof StandardSQL))
                            */
                            Util.exit("Tuple "+whereClause.toString()+ "to be deleted but not found... weird...");
                        }
                        
                    }
                    else { // update-tuple
                        // set-query
                        String setClause = new String();
                        String tmpEntry,tmpValueAfter,tmpValueBefore;
                        boolean includeInSetClause;
                        boolean firstEntry = true;
                        for (int i = 0; i<updD.before.getAttributeNames().size(); i++) {
                            tmpEntry=new String();
                            if (!firstEntry) tmpEntry += ", ";
                            tmpEntry += updD.after.getAttributeNames().elementAt(i) + "=";
                            tmpValueAfter = (String)((Tuple)updD.after.getTuples().elementAt(0)).getVector().elementAt(i);
                            tmpValueBefore = (String)((Tuple)updD.before.getTuples().elementAt(0)).getVector().elementAt(i);
                            includeInSetClause=true;
                            // we need to set only if old value!=new
                            // value and have to make sure we don't
                            // have quotes if the new value is null
                            if (tmpValueAfter==null) {
                                if (tmpValueBefore!=null) tmpValueAfter = "null";
                                else includeInSetClause = false;
                            }
                            else {
                                if (!tmpValueAfter.equals(tmpValueBefore)) 
                                    tmpValueAfter = "'"+tmpValueAfter+"'";
                                else includeInSetClause = false;
                            }
                            if (includeInSetClause) {
                                setClause += tmpEntry + tmpValueAfter;
                                firstEntry = false;
                            }
                        }
                    // now whereClause contains a where clause for all
                    // tuple values in the before-tuple and setClause
                    // contains all values to be set which are
                    // different from before

                        query = "update "
                            + outputTableName
                            + " set "
                            + setClause
                            + " where "
                            + whereClause;
                        int result = localDB.executeSQLUpdate(query);
                        if (result==0) {
                            // if we have a standardsql-node, this
                            // simply means the tuple is projected
                            // out, otherwise it's an error:
                            if (!(this instanceof StandardSQL))
                                Util.exit("Tuple "+whereClause.toString()+ "to be deleted but not found... weird...");
                        }
                    } // else from if (updD.after==null)
                } // else from if(updD.before==null)
            }  // if upd instanceof DataUpdate
            else {
                updS = (SchemaUpdate)upd;
                switch (updS.type) {
                case SchemaUpdate.ADD:
                    if (updS.attribute==null) { // relation-update
                        // we don't know the schema here so we
                        // crash. Makes only sense for the Unite
                        // operator (override?)
                        Util.exit("add-Relation doesn't make sense except for Unite and it's not implemented there");
                    }
                    else {//attribute-update
                        System.out.println("WARNING: adding attribute with dummy data type char(32)");
                        localDB.addAttribute(updS.relation,updS.attribute,"CHAR",31);
                    }
                    break;
                case SchemaUpdate.DELETE:
                    if (updS.attribute==null) { // relation-update
                        localDB.deleteRelation(updS.relation,false); // non-silent delete
                    }
                    else {//attribute-update
                        localDB.deleteAttribute(updS.relation,updS.attribute);
                    }
                    break;
                case SchemaUpdate.RENAME:
                    if (updS.attribute==null) { // relation-update
                        localDB.renameRelation(updS.relation,updS.newName);
                    }
                    else {//attribute-update
                        localDB.renameAttribute(updS.relation,updS.attribute,updS.newName);
                    }
                    break;
                default:
                    Util.exit("invalid schema update");
                }
            }
        }
    }


    /** every operator must know how to propagate incremental updates
     *   of any kind. This method should be abstract in this class as
     *   soon as the code is done. The function returns a vector of
     *   Updates that were generated in the propagation. Some
     *   propagations can not be expressed in this way. */
    
    /*abstract*/ Vector propagateUpdate(Update upd){
        System.err.println("Something is weird. SchemaSQLOperator::propagateUpdate() should not be called.");
        return null;
    }


    //============================================================
    //============================================================
    // protected helper functions for incremental updates
    //============================================================
    //============================================================

    protected int findPivotCount(String pivotName) {
        Tuple temp;
        // grrrrr.... this cost me two hours of searching
        pivotName = pivotName.trim();
        int result=0;
        if (pivotData == null)
            Util.exit("no pivot data table available yet.");
        else
            for (Enumeration enum=pivotData.getTuples().elements();
                 enum.hasMoreElements();) {
                temp = (Tuple)enum.nextElement();
                if (pivotName.equalsIgnoreCase(
                                               ((String)temp.getVector().elementAt(0)).trim()
                                               )
                    ) 
                    result = (Integer.parseInt((String)temp.getVector().elementAt(1)));
            }
        return result;
    }

    /** increments the count for a pivot element in this operator's
        (materialized) pivot vector. The pivot vector (pivot table)
        serves as a store for pivot elements (it's the paper's A^*)
        and is actually not really necessary since we query the output
        table every time anyway. But since this is not something we're
        proud of, we will just keep having the pivot vector around. */

    protected boolean incPivotCount(String pivotName) {
        return changePivotCount(pivotName, +1);
    }

    /** decrement pivot count. @see incPivotCount(String) */
    protected boolean decPivotCount(String pivotName) {
        return changePivotCount(pivotName, -1);
    }
    
    /** change the pivot count for a certain element by a certain
        amount. Should be called only from @see incPivotCount(String)
        and @see decPivotCount(String) */
    private boolean changePivotCount(String pivotName, int increment) {
        Tuple temp;
        pivotName = pivotName.trim();
        int oldPivotCount;
        if (pivotData == null)
            Util.exit("no pivot data table available yet.");
        else
            for (Enumeration enum=pivotData.getTuples().elements();
                 enum.hasMoreElements();) {
                temp = (Tuple)enum.nextElement();
                if (pivotName.equalsIgnoreCase(
                                               ((String)temp.getVector().elementAt(0)).trim()
                                               )
                    ) {
                    oldPivotCount = Integer.parseInt((String)temp.getVector().elementAt(1));
                    temp.getVector().setElementAt(new Integer(oldPivotCount+increment).toString(),1);
                    return true;
                }
            }
        Util.exit("trying to increase pivotCount for a non-existing pivot");
        return false;
    }

    
    /** add a pivot string to the pivot vector. @return false if pivot exists */
    protected boolean addPivot(String pivotName) {
        Tuple temp;
        pivotName = pivotName.trim();
        if (pivotData == null)
            Util.exit("no pivot data table available yet.");
        else
            for (Enumeration enum=pivotData.getTuples().elements();
                 enum.hasMoreElements();) {
                temp = (Tuple)enum.nextElement();
                if (pivotName.equalsIgnoreCase(
                                               ((String)temp.getVector().elementAt(0)).trim()
                                               )) 
{
                    Util.exit("trying to add existing tuple to pivotTable");
                    return false;
                }
            }
        Vector tmpVector = new Vector(2);
        tmpVector.addElement(pivotName);
        tmpVector.addElement("1");
        temp = new Tuple(pivotData,tmpVector);
        pivotData.addTuple(temp);
        return true;
    }
    
    /** delete a pivot string from the pivot vector. @return false if
        pivot doesn't exist. */
    protected boolean delPivot(String pivotName) {
        pivotName = pivotName.trim();
        Vector tmpVector = new Vector(2);
        tmpVector.addElement(pivotName);
        tmpVector.addElement(new Integer(findPivotCount(pivotName)).toString());
        Tuple tmpTuple = new Tuple(pivotData,tmpVector);
        return (pivotData.removeTuple(tmpTuple,1)==1);
    }

    /** this function builds an update query for data-changes and
        executes it. It returns the table object with the result.
        @param tableName is the name of the table to be queried
        @param relSchema is a vector of strings holding the attribute
        names to be added to the where-clause
        @param updateTable is a table with attribute names and values
        (the update to be constructed). */
    protected Table executeUpdateQuery(String tableName, Vector schema, Table updateTable) {
        String currentName;
        String query = "select * from "
            + tableName;
        String tmpValue;
        boolean firstTime=true;
        for (Enumeration enum=schema.elements(); 
             enum.hasMoreElements();) {
            currentName = (String)enum.nextElement();
            if (firstTime) {
                query += " where ";
                firstTime = false;
            }
            else
                query += " and ";
            tmpValue = (String)updateTable.getColumn(currentName).elementAt(0);
            if (tmpValue==null)
                query += currentName+ " is null ";
            else
                query += currentName
                    + " = '"
                    +
                    tmpValue
                    + "'";
        }
        Table tmpTable = localDB.executeViewQuery(query);
        tmpTable.setRelName("DUMMY");
        
        return tmpTable;
    }
    
    /** this function translates all tuples in <I>delta</I> into
        delete-tuple updates in table <i>tableName</i> */

    /* package private */ Vector tableToDeleteUpdates(String tableName, Table
    delta) {
        Vector result = new Vector();
        DataUpdate tmpUpdate;
        Table tmpTable;
        // now iterate through table and construct new DataUpdate for each tuple
        if (delta==null) return null;
        if (delta.getTuples().size()==0) return null;
        
        for (Enumeration enum=delta.getTuples().elements();
             enum.hasMoreElements();) {
            tmpTable = new Table();
            tmpTable.setRelName("DUMMY");
            tmpTable.setAttributeNames(delta.getAttributeNames());
            tmpTable.addTuple((Tuple)enum.nextElement());
            tmpUpdate = new DataUpdate(tableName,tmpTable,null);
            result.addElement(tmpUpdate);
        }
        return result;
    }
    
    
}    
