package algebraTree;
import java.util.*;

import DSRG.MW.*;

/** A schema update. This is logical only - the physical update should
 *   not have anything to do with this class!  */

public class SchemaUpdate extends Update {
    /** constants for the three types of updates */
    public final static int ADD=1;
    public final static int DELETE=2;
    public final static int RENAME=3;


    /** this is for the toString method only! */
    public final static String[] typeNames = {"ERROR","ADD","DELETE","RENAME"};

    
    /** The (name of the) object that is affected. If "attribute" is
        null, we have a relation schema change. The relation name is
        stored in the superclass ("Update") */
    public String attribute =  null;
    public int type = -1;
    /** for rename-schema-changes, we need to store the new name */
    public String newName =  null;

    /** the constructor for a relation add/delete */
    public SchemaUpdate(String relationP, int typeP) {
        relation = relationP.toUpperCase();
        attribute = null;
        type = typeP;
        if (type!=ADD && type != DELETE && type != RENAME)
            Util.exit("Invalid schema change");
    }
    
    /** the constructor for attribute add/del as well as relation-rename! */
    public SchemaUpdate(String p1, String p2, int typeP) {
        this(p1,typeP);
        if (typeP == RENAME) {
            // this is a relation-rename. Always in capitals!
            newName = p2.toUpperCase();
        }
        else {
            attribute = p2.toUpperCase();
        }
    }
    
    /** the constructor for attribute-rename */
    public SchemaUpdate(String relationP, String attributeP, String newNameP, int typeP) {
        this(relationP,typeP);
        if(typeP!=RENAME) 
            Util.exit("Invalid schema update");
        else {
            attribute = attributeP.toUpperCase();
            newName = newNameP.toUpperCase(); // always in capitals!
        }
    }
    
    /** copy constructor for update propagation */
    public SchemaUpdate(SchemaUpdate old) {
        relation = old.relation;
        attribute = old.attribute;
        newName = old.newName;
        type = old.type;
    }

    public String toString() {
        return "SchemaUpdate -- "+typeNames[type]+": Rel: "+relation+" Attribute: "+attribute+" newName: "+newName;
    }
}    
