package algebraTree;
import java.util.*;
import DSRG.MW.*;

public class Unite extends SchemaSQLOperator {
    Vector relationNames = new Vector(5);
    String pivotName;

    /** the vector of all attribute names of the input relation */
    Vector attNames;

    public Unite(DBManager localDB, DBManager sourceDB, 
                 Vector relNames, String pivot) {
        super(localDB,true); // UNITE is always a leaf node (really?)
        if (!setSourceDB(sourceDB)) {
            System.out.println("Unite operator must be leaf.");
            System.exit(0);
        }
        relationNames = relNames;
        pivotName = pivot;
    }
    
    public String getPivotName() {
        return pivotName;
    }
    
    protected Vector computeLocalQueries() {
        Vector result = new Vector();
        String selectClause = new String();
        // get all attribute names for one relation. This is assuming
        // that all input relations have the same schema
        attNames = sourceDB.getAttributeNames
            ((String)relationNames.elementAt(0));

        for (Enumeration enum=attNames.elements();
             enum.hasMoreElements();) 
            selectClause += (", "+(String)enum.nextElement());
        
        // now generate one select statement for each table
        String currentName;
        String uniteQuery = new String();
        boolean firstTime = true;
        for (Enumeration enum=relationNames.elements();
             enum.hasMoreElements();) {
            currentName = (String)enum.nextElement();
            if (firstTime)
                firstTime=false;
            else
                uniteQuery = uniteQuery + " UNION ";
            uniteQuery += (
                           "select '" 
                           + currentName
                           + "' as "
                           + "\""+pivotName+"\""
                           + " "
                           + selectClause
                           + " from "
                           + currentName
                           );
            
        }
        result.addElement(uniteQuery);
        if (Main.DEBUG) System.out.println("UNITE produced: "+result.toString());
        return result;
    }






    /** propagate an update through this operator */
    /* package private */Vector propagateUpdate(Update upd) {
        if (Main.DEBUG) 
            System.err.println(this.toString()+": Trying to propagate: "+upd);
 
       // if update is not for me, return null
        Vector result = new Vector();
        String updPivot;
        if (relationNames.indexOf(upd.relation.toUpperCase())==-1)
            return null;

        if (upd instanceof DataUpdate) {
            DataUpdate updD = (DataUpdate)upd;
            // a data update
            if (updD.before==null) { // add-tuple
                // the output schema is the input schema plus one
                // attribute called pivotName. We'll add it at
                // position 0
                Vector outputSchema=(Vector)attNames.clone();
                outputSchema.insertElementAt(pivotName,0);
                Table insertTable = new Table();
                insertTable.setRelName("DUMMY");
                insertTable.setAttributeNames(outputSchema);
                Vector insertValues = new Vector(outputSchema.size());
                // add name of relation in position of the pivotName
                insertValues.addElement(updD.relation);
                //then, copy relevant values from old update into new
                //update template. Instead of relying on the order of
                //attributes, match by attribute name
                Vector dataValuesFromUpdate = ((Tuple)updD.after.getTuples().elementAt(0)).getVector();
                for (int i = 1; i < outputSchema.size(); i++)
                    insertValues.addElement(dataValuesFromUpdate.elementAt(updD.after.getAttributeNames().indexOf(((String)outputSchema.elementAt(i)).toUpperCase())));

                // now, create a tuple
                Tuple insertTuple = new Tuple(insertTable, insertValues);
                // and create a table with it. This will go to the Update.
                insertTable.addTuple(insertTuple);
                DataUpdate tmpUpdate;
                tmpUpdate = new DataUpdate(outputTableName,null,insertTable);
                result.addElement(tmpUpdate);
            }
            else if (updD.after==null) {// delete tuple

                // first, find all output tuples that match in
                // everything but the relation name. We'll get at most
                // as many tuples as there are input relations, so
                // that shouldn't be too bad.
                Table deleteTable = executeUpdateQuery(outputTableName, attNames, updD.before);
                //now, we go through them and find the one that
                //matches the relation name. That's our only update.
                Tuple tmpTuple;
                DataUpdate tmpUpdate=null;
                Table tmpTable = new Table();
                tmpTable.setRelName("DUMMY");
                Vector outputSchema = new Vector(attNames);
                outputSchema.insertElementAt(pivotName,0);
                tmpTable.setAttributeNames(outputSchema);
                for (Enumeration enum = deleteTable.getTuples().elements();
                     enum.hasMoreElements();) {
                    tmpTuple = (Tuple)enum.nextElement();
                    String tmpString = (String)tmpTuple.getVector().elementAt(deleteTable.getAttributeNames().indexOf(pivotName));
                    if (tmpString.equalsIgnoreCase(updD.relation)) {
                        tmpTable.addTuple(tmpTuple);
                        tmpUpdate = new DataUpdate(outputTableName,tmpTable,null);
                        break;
                    }
                }
                // if tuple doesn't exist, we'll get a null 
                if (tmpUpdate != null)
                    result.addElement(tmpUpdate);
                else 
                    Util.exit("Trying to delete non-existing tuple: "+updD.before.toString()+" from "+updD.relation);
            }
            else { // update-tuple

                // this recursive solution is simpler but less
                // efficient. May change sometime (yeah,right!!)
                // 082700 koeller

                // recursive call. Make sure we don't have update-tuple-changes here!!!!
                // add-tuple
                result = this.propagateUpdate(new DataUpdate(updD.relation,null,updD.after));
                // delete-tuple
                result = QENode.mergeVectors(result,this.propagateUpdate(new DataUpdate(updD.relation,updD.before,null)));
            }
        }
        else {
            // a schema change.
            SchemaUpdate updS = (SchemaUpdate)upd;
            SchemaUpdate tmpSchemaUpdate;
            if (updS.attribute!=null) {
                // we will get schema changes for all relations at the
                // same time. Therefore, we will only look at updates that
                // came from the first relation in "relationNames"
                if (!updS.relation.equalsIgnoreCase((String)relationNames.elementAt(0)))
                    return null;
                // else an attribute update. Same propagation for all
                // attribute updates
                tmpSchemaUpdate = new SchemaUpdate(updS);
                tmpSchemaUpdate.relation = new String(outputTableName);
                result.addElement(tmpSchemaUpdate);
            }
            else {
                switch (updS.type) {
                case SchemaUpdate.ADD:
                    // do nothing... makes no sense
                    break;
                case SchemaUpdate.DELETE:
                    // first, find all output tuples that match the
                    // relation name. 
                    Table deleteTable = 
                        localDB.executeViewQuery("select * from "
                                                  +outputTableName
                                                  +" where "
                                                  +pivotName.toUpperCase()
                                                  +" = '"
                                                  +updS.relation
                                                  +"'");
                    
                    // now we translate this into a bunch of updates
                    Tuple tmpTuple; Table tmpTable;
                    DataUpdate tmpUpdate;
                    Table templateTable = new Table();
                    templateTable.setRelName("DUMMY");
                    Vector outputSchema = new Vector(attNames);
                    outputSchema.insertElementAt(pivotName,0);
                    templateTable.setAttributeNames(outputSchema);
                    for (Enumeration enum = deleteTable.getTuples().elements();
                         enum.hasMoreElements();) {
                        tmpTuple = (Tuple)enum.nextElement();
                        // set first column's values to relation name
                        tmpTuple.getVector().setElementAt(updS.relation,0);
                        tmpTable = new Table(templateTable);
                        tmpTable.addTuple(tmpTuple);
                        tmpUpdate = new DataUpdate(outputTableName,tmpTable,null);
                        result.addElement(tmpUpdate);
                    }
                    relationNames.removeElement(updS.relation.toUpperCase());
                    break;
                case SchemaUpdate.RENAME:
                    relationNames.setElementAt(updS.newName,relationNames.indexOf(updS.relation.toUpperCase()));
                    break;
                }
            }
        }
        if (Main.DEBUG) System.out.println("Incremental Update Propagation. UNITE gave us: "+result.toString());
        // the QENode above this must execute these updates!!!
        return result;
    }








}
