//
// Generated by JTB 1.2.1
//
package schemaSQLParser.visitor;

import schemaSQLParser.syntaxtree.*;
import java.util.*;

/**
 * A skeleton output formatter for your language grammar.  Using the
 * add() method along with force(), indent(), and outdent(), you can
 * easily specify how this visitor will format the given syntax tree.
 * See the JTB documentation for more details.
 *
 * Pass your syntax tree to this visitor, and then to the TreeDumper
 * visitor in order to "pretty print" your tree.
 */
public class TreeFormatter extends DepthFirstVisitor {
   private Vector cmdQueue = new Vector();
   private boolean lineWrap;
   private int wrapWidth;
   private int indentAmt;
   private int curLine = 1;
   private int curColumn = 1;
   private int curIndent = 0;

   /**
    * The default constructor assumes an indentation amount of 3 spaces
    * and no line-wrap.  You may alternately use the other constructor to
    * specify your own indentation amount and line width.
    */
   public TreeFormatter() { this(3, 0); }

   /**
    * This constructor accepts an indent amount and a line width which is
    * used to wrap long lines.  If a token's beginColumn value is greater
    * than the specified wrapWidth, it will be moved to the next line and
    * indented one extra level.  To turn off line-wrapping, specify a
    * wrapWidth of 0.
    *
    * @param   indentAmt   Amount of spaces per indentation level.
    * @param   wrapWidth   Wrap lines longer than wrapWidth.  0 for no wrap.
    */
   public TreeFormatter(int indentAmt, int wrapWidth) {
      this.indentAmt = indentAmt;
      this.wrapWidth = wrapWidth;

      if ( wrapWidth > 0 )
         lineWrap = true;
      else
         lineWrap = false;
   }

   /**
    * Accepts a NodeListInterface object and performs an optional format
    * command between each node in the list (but not after the last node).
    */
   protected void processList(NodeListInterface n) {
      processList(n, null);
   }

   protected void processList(NodeListInterface n, FormatCommand cmd) {
      for ( Enumeration e = n.elements(); e.hasMoreElements(); ) {
         ((Node)e.nextElement()).accept(this);
         if ( cmd != null && e.hasMoreElements() )
            cmdQueue.addElement(cmd);
      }
   }

   /**
    * A Force command inserts a line break and indents the next line to
    * the current indentation level.  Use "add(force());".
    */
   protected FormatCommand force() { return force(1); }
   protected FormatCommand force(int i) {
      return new FormatCommand(FormatCommand.FORCE, i);
   }

   /**
    * An Indent command increases the indentation level by one (or a
    * user-specified amount).  Use "add(indent());".
    */
   protected FormatCommand indent() { return indent(1); }
   protected FormatCommand indent(int i) {
      return new FormatCommand(FormatCommand.INDENT, i);
   }

   /**
    * An Outdent command is the reverse of the Indent command: it reduces
    * the indentation level.  Use "add(outdent());".
    */
   protected FormatCommand outdent() { return outdent(1); }
   protected FormatCommand outdent(int i) {
      return new FormatCommand(FormatCommand.OUTDENT, i);
   }

   /**
    * A Space command simply adds one or a user-specified number of
    * spaces between tokens.  Use "add(space());".
    */
   protected FormatCommand space() { return space(1); }
   protected FormatCommand space(int i) {
      return new FormatCommand(FormatCommand.SPACE, i);
   }

   /**
    * Use this method to add FormatCommands to the command queue to be
    * executed when the next token in the tree is visited.
    */
   protected void add(FormatCommand cmd) {
      cmdQueue.addElement(cmd);
   }

   /**
    * Executes the commands waiting in the command queue, then inserts the
    * proper location information into the current NodeToken.
    *
    * If there are any special tokens preceding this token, they will be
    * given the current location information.  The token will follow on
    * the next line, at the proper indentation level.  If this is not the
    * behavior you want from special tokens, feel free to modify this
    * method.
    */
   public void visit(NodeToken n) {
      for ( Enumeration e = cmdQueue.elements(); e.hasMoreElements(); ) {
         FormatCommand cmd = (FormatCommand)e.nextElement();
         switch ( cmd.getCommand() ) {
         case FormatCommand.FORCE :
            curLine += cmd.getNumCommands();
            curColumn = curIndent + 1;
            break;
         case FormatCommand.INDENT :
            curIndent += indentAmt * cmd.getNumCommands();
            break;
         case FormatCommand.OUTDENT :
            if ( curIndent >= indentAmt )
               curIndent -= indentAmt * cmd.getNumCommands();
            break;
         case FormatCommand.SPACE :
            curColumn += cmd.getNumCommands();
            break;
         default :
            throw new TreeFormatterException(
               "Invalid value in command queue.");
         }
      }

      cmdQueue.removeAllElements();

      //
      // Handle all special tokens preceding this NodeToken
      //
      if ( n.numSpecials() > 0 )
         for ( Enumeration e = n.specialTokens.elements();
               e.hasMoreElements(); ) {
            NodeToken special = (NodeToken)e.nextElement();

            //
            // -Place the token.
            // -Move cursor to next line after the special token.
            // -Don't update curColumn--want to keep current indent level.
            //
            placeToken(special, curLine, curColumn);
            curLine = special.endLine + 1;
         }

      placeToken(n, curLine, curColumn);
      curLine = n.endLine;
      curColumn = n.endColumn;
   }

   /**
    * Inserts token location (beginLine, beginColumn, endLine, endColumn)
    * information into the NodeToken.  Takes into account line-wrap.
    * Does not update curLine and curColumn.
    */
   private void placeToken(NodeToken n, int line, int column) {
      int length = n.tokenImage.length();

      //
      // Find beginning of token.  Only line-wrap for single-line tokens
      //
      if ( !lineWrap || n.tokenImage.indexOf('\n') != -1 ||
           column + length <= wrapWidth )
         n.beginColumn = column;
      else {
         ++line;
         column = curIndent + indentAmt + 1;
         n.beginColumn = column;
      }

      n.beginLine = line;

      //
      // Find end of token; don't count \n if it's the last character
      //
      for ( int i = 0; i < length; ++i ) {
         if ( n.tokenImage.charAt(i) == '\n' && i < length - 1 ) {
            ++line;
            column = 1;
         }
         else
            ++column;
      }

      n.endLine = line;
      n.endColumn = column;
   }

   //
   // User-generated visitor methods below
   //

   /**
    * nodeChoice -> ProcedureDeclaration()
    *       | FunctionDeclaration()
    *       | SequenceOfStatements()
    */
   public void visit(CompilationUnit n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "DECLARE"
    * declarations -> Declarations()
    */
   public void visit(DeclarationSection n) {
      n.nodeToken.accept(this);
      n.declarations.accept(this);
   }

   /**
    * nodeList -> ( ( IdentifierDeclaration() | CursorDeclaration() | PragmaDeclaration() ) ";" )+
    * nodeListOptional -> ( ProcedureDeclaration() | FunctionDeclaration() )*
    */
   public void visit(Declarations n) {
      processList(n.nodeList);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeToken -> <S_IDENTIFIER>
    * nodeChoice -> ( ConstantDeclaration() | VariableDeclaration() | ExceptionDeclaration() )
    */
   public void visit(IdentifierDeclaration n) {
      n.nodeToken.accept(this);
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "CURSOR"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeOptional -> [ "(" ParameterList() ")" ]
    * nodeToken2 -> "IS"
    * selectStatement -> SelectStatement()
    */
   public void visit(CursorDeclaration n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken2.accept(this);
      n.selectStatement.accept(this);
   }

   /**
    * nodeToken -> "PROCEDURE"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeOptional -> [ "(" ParameterList() ")" ]
    * nodeChoice -> ( ";" | "IS" ProcedureBody() )
    */
   public void visit(ProcedureDeclaration n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeChoice.accept(this);
   }

   /**
    * nodeOptional -> [ Declarations() ]
    * beginEndBlock -> BeginEndBlock()
    */
   public void visit(ProcedureBody n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.beginEndBlock.accept(this);
   }

   /**
    * nodeToken -> "FUNCTION"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeOptional -> [ "(" ParameterList() ")" ]
    * nodeToken2 -> "RETURN"
    * typeDeclaration -> TypeDeclaration()
    * nodeChoice -> ( ";" | "IS" FunctionBody() )
    */
   public void visit(FunctionDeclaration n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken2.accept(this);
      n.typeDeclaration.accept(this);
      n.nodeChoice.accept(this);
   }

   /**
    * nodeOptional -> [ Declarations() ]
    * beginEndBlock -> BeginEndBlock()
    */
   public void visit(FunctionBody n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.beginEndBlock.accept(this);
   }

   /**
    * nodeToken -> "PRAGMA"
    * nodeToken1 -> "EXCEPTION_INIT"
    * nodeToken2 -> "("
    * numOrID -> NumOrID()
    * nodeToken3 -> ","
    * numOrID1 -> NumOrID()
    * nodeToken4 -> ")"
    */
   public void visit(PragmaDeclaration n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
      n.numOrID.accept(this);
      n.nodeToken3.accept(this);
      n.numOrID1.accept(this);
      n.nodeToken4.accept(this);
   }

   /**
    * typeDeclaration -> TypeDeclaration()
    * nodeOptional -> [ "NOT" "NULL" ]
    * nodeOptional1 -> [ ( ":=" | "DEFAULT" ) PlSqlExpression() ]
    */
   public void visit(VariableDeclaration n) {
      n.typeDeclaration.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
   }

   /**
    * nodeToken -> "CONSTANT"
    * typeDeclaration -> TypeDeclaration()
    * nodeOptional -> [ "NOT" "NULL" ]
    * nodeChoice -> ( ":=" | "DEFAULT" )
    * plSqlExpression -> PlSqlExpression()
    */
   public void visit(ConstantDeclaration n) {
      n.nodeToken.accept(this);
      n.typeDeclaration.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeChoice.accept(this);
      n.plSqlExpression.accept(this);
   }

   /**
    * nodeChoice -> BasicDataTypeDeclaration()
    *       | ( <S_IDENTIFIER> ( "%TYPE" | "%ROWTYPE" ) )
    *       | TableColumn() "%TYPE"
    *       | <S_IDENTIFIER>
    */
   public void visit(TypeDeclaration n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeChoice -> ( "CHAR" | "VARCHAR" | "VARCHAR2" | "INTEGER" | "NUMBER" | "NATURAL" | "REAL" | "FLOAT" ) [ "(" <S_NUMBER> [ "," <S_NUMBER> ] ")" ]
    *       | "DATE"
    *       | "BINARY_INTEGER"
    *       | "BOOLEAN"
    */
   public void visit(BasicDataTypeDeclaration n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "EXCEPTION"
    */
   public void visit(ExceptionDeclaration n) {
      n.nodeToken.accept(this);
   }

   /**
    * nodeToken -> "BEGIN"
    * sequenceOfStatements -> SequenceOfStatements()
    * nodeOptional -> [ ExceptionBlock() ]
    * nodeToken1 -> "END"
    * nodeOptional1 -> [ <S_IDENTIFIER> ]
    * nodeToken2 -> ";"
    */
   public void visit(BeginEndBlock n) {
      n.nodeToken.accept(this);
      n.sequenceOfStatements.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken1.accept(this);
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken2.accept(this);
   }

   /**
    * nodeList -> ( PLSQLStatement() )+
    */
   public void visit(SequenceOfStatements n) {
      processList(n.nodeList);
   }

   /**
    * nodeToken -> "EXCEPTION"
    * nodeList -> ( ExceptionHandler() )+
    */
   public void visit(ExceptionBlock n) {
      n.nodeToken.accept(this);
      processList(n.nodeList);
   }

   /**
    * nodeToken -> "WHEN"
    * nodeChoice -> ( <S_IDENTIFIER> ( "OR" <S_IDENTIFIER> )* | "OTHERS" )
    * nodeToken1 -> "THEN"
    * sequenceOfStatements -> SequenceOfStatements()
    */
   public void visit(ExceptionHandler n) {
      n.nodeToken.accept(this);
      n.nodeChoice.accept(this);
      n.nodeToken1.accept(this);
      n.sequenceOfStatements.accept(this);
   }

   /**
    * nodeChoice -> SubroutineCall()
    *       | AssignmentStatement()
    *       | ExitStatement()
    *       | GotoStatement()
    *       | IfStatement()
    *       | LoopStatement()
    *       | NullStatement()
    *       | RaiseStatement()
    *       | ReturnStatement()
    *       | SQLStatement()
    *       | LabelDeclaration()
    *       | [ DeclarationSection() ] BeginEndBlock()
    */
   public void visit(PLSQLStatement n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "<<"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> ">>"
    */
   public void visit(LabelDeclaration n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeChoice -> CloseStatement()
    *       | CommitStatement()
    *       | DeleteStatement()
    *       | FetchStatement()
    *       | InsertStatement()
    *       | LockTableStatement()
    *       | OpenStatement()
    *       | RollbackStatement()
    *       | SavepointStatement()
    *       | QueryStatement()
    *       | SetTransactionStatement()
    *       | UpdateStatement()
    */
   public void visit(SQLStatement n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> <S_IDENTIFIER>
    * nodeOptional -> [ "." <S_IDENTIFIER> ]
    * nodeOptional1 -> [ "(" Arguments() ")" ]
    * nodeToken1 -> ";"
    */
   public void visit(SubroutineCall n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeChoice -> ( ( <S_IDENTIFIER> [ "." <S_IDENTIFIER> ] ":=" PlSqlExpression() ) | ( <S_BIND> ":=" PlSqlExpression() ) )
    * nodeToken -> ";"
    */
   public void visit(AssignmentStatement n) {
      n.nodeChoice.accept(this);
      n.nodeToken.accept(this);
   }

   /**
    * nodeToken -> "EXIT"
    * nodeOptional -> [ <S_IDENTIFIER> ]
    * nodeOptional1 -> [ "WHEN" PlSqlExpression() ]
    * nodeToken1 -> ";"
    */
   public void visit(ExitStatement n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeToken -> "GOTO"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> ";"
    */
   public void visit(GotoStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeToken -> "IF"
    * plSqlExpression -> PlSqlExpression()
    * nodeToken1 -> "THEN"
    * sequenceOfStatements -> SequenceOfStatements()
    * nodeListOptional -> ( "ELSIF" PlSqlExpression() "THEN" SequenceOfStatements() )*
    * nodeOptional -> [ "ELSE" SequenceOfStatements() ]
    * nodeToken2 -> "END"
    * nodeToken3 -> "IF"
    * nodeOptional1 -> [ <S_IDENTIFIER> ]
    * nodeToken4 -> ";"
    */
   public void visit(IfStatement n) {
      n.nodeToken.accept(this);
      n.plSqlExpression.accept(this);
      n.nodeToken1.accept(this);
      n.sequenceOfStatements.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken2.accept(this);
      n.nodeToken3.accept(this);
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken4.accept(this);
   }

   /**
    * nodeChoice -> NormalLoop()
    *       | WhileLoop()
    *       | ForLoop()
    */
   public void visit(LoopStatement n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "LOOP"
    * sequenceOfStatements -> SequenceOfStatements()
    * nodeToken1 -> "END"
    * nodeToken2 -> "LOOP"
    * nodeOptional -> [ <S_IDENTIFIER> ]
    * nodeToken3 -> ";"
    */
   public void visit(NormalLoop n) {
      n.nodeToken.accept(this);
      n.sequenceOfStatements.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken3.accept(this);
   }

   /**
    * nodeToken -> "WHILE"
    * plSqlExpression -> PlSqlExpression()
    * normalLoop -> NormalLoop()
    */
   public void visit(WhileLoop n) {
      n.nodeToken.accept(this);
      n.plSqlExpression.accept(this);
      n.normalLoop.accept(this);
   }

   /**
    * nodeChoice -> NumericForLoop()
    *       | CursorForLoop()
    */
   public void visit(ForLoop n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "FOR"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> "IN"
    * nodeOptional -> [ "REVERSE" ]
    * plSqlSimpleExpression -> PlSqlSimpleExpression()
    * nodeToken3 -> ".."
    */
   public void visit(NumericForLoopLookahead n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.plSqlSimpleExpression.accept(this);
      n.nodeToken3.accept(this);
   }

   /**
    * nodeToken -> "FOR"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> "IN"
    * nodeOptional -> [ "REVERSE" ]
    * plSqlSimpleExpression -> PlSqlSimpleExpression()
    * nodeToken3 -> ".."
    * plSqlSimpleExpression1 -> PlSqlSimpleExpression()
    * normalLoop -> NormalLoop()
    */
   public void visit(NumericForLoop n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.plSqlSimpleExpression.accept(this);
      n.nodeToken3.accept(this);
      n.plSqlSimpleExpression1.accept(this);
      n.normalLoop.accept(this);
   }

   /**
    * nodeToken -> "FOR"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> "IN"
    * nodeChoice -> ( <S_IDENTIFIER> [ "(" Arguments() ")" ] | "(" SelectStatement() ")" )
    * normalLoop -> NormalLoop()
    */
   public void visit(CursorForLoop n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
      n.nodeChoice.accept(this);
      n.normalLoop.accept(this);
   }

   /**
    * nodeToken -> "NULL"
    * nodeToken1 -> ";"
    */
   public void visit(NullStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
   }

   /**
    * nodeToken -> "RAISE"
    * nodeOptional -> [ <S_IDENTIFIER> ]
    * nodeToken1 -> ";"
    */
   public void visit(RaiseStatement n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeToken -> "RETURN"
    * nodeOptional -> [ PlSqlExpression() ]
    * nodeToken1 -> ";"
    */
   public void visit(ReturnStatement n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeToken -> "CLOSE"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> ";"
    */
   public void visit(CloseStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeToken -> "COMMIT"
    * nodeOptional -> [ "WORK" ]
    * nodeOptional1 -> [ "COMMENT" <S_CHAR_LITERAL> ]
    * nodeToken1 -> ";"
    */
   public void visit(CommitStatement n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeToken -> "FETCH"
    * nodeSequence -> ( <S_IDENTIFIER> )
    * nodeToken1 -> "INTO"
    * nodeChoice -> ( <S_IDENTIFIER> | <S_BIND> )
    * nodeListOptional -> ( "," ( <S_IDENTIFIER> | <S_BIND> ) )*
    * nodeToken2 -> ";"
    */
   public void visit(FetchStatement n) {
      n.nodeToken.accept(this);
      n.nodeSequence.accept(this);
      n.nodeToken1.accept(this);
      n.nodeChoice.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
      n.nodeToken2.accept(this);
   }

   /**
    * nodeToken -> "LOCK"
    * nodeToken1 -> "TABLE"
    * tableReference -> TableReference()
    * nodeListOptional -> ( "," TableReference() )*
    * nodeToken2 -> "IN"
    * lockMode -> LockMode()
    * nodeToken3 -> "MODE"
    * nodeOptional -> [ "NOWAIT" ]
    * nodeToken4 -> ";"
    */
   public void visit(LockTableStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.tableReference.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
      n.nodeToken2.accept(this);
      n.lockMode.accept(this);
      n.nodeToken3.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken4.accept(this);
   }

   /**
    * nodeToken -> "OPEN"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeOptional -> [ "(" Arguments() ")" ]
    * nodeToken2 -> ";"
    */
   public void visit(OpenStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken2.accept(this);
   }

   /**
    * nodeToken -> "ROLLBACK"
    * nodeOptional -> [ "WORK" ]
    * nodeOptional1 -> [ "TO" [ "SAVEPOINT" ] <S_IDENTIFIER> ]
    * nodeOptional2 -> [ "COMMENT" <S_CHAR_LITERAL> ]
    * nodeToken1 -> ";"
    */
   public void visit(RollbackStatement n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      if ( n.nodeOptional2.present() ) {
         n.nodeOptional2.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeToken -> "SET"
    * nodeToken1 -> "TRANSACTION"
    * nodeChoice -> ( ( "READ" ( "ONLY" | "WRITE" ) ) | ( "USE" "ROLLBACK" "SEGMENT" <S_IDENTIFIER> ) )
    * nodeToken2 -> ";"
    */
   public void visit(SetTransactionStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeChoice.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeChoice -> ( "ROW" ( "SHARE" | "EXCLUSIVE" ) )
    *       | ( "SHARE" [ "UPDATE" | ( "ROW" "EXCLUSIVE" ) ] )
    *       | ( "EXCLUSIVE" )
    */
   public void visit(LockMode n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "SAVEPOINT"
    * nodeToken1 -> <S_IDENTIFIER>
    * nodeToken2 -> ";"
    */
   public void visit(SavepointStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeToken -> "UPDATE"
    * tableReference -> TableReference()
    * nodeOptional -> [ <S_IDENTIFIER> ]
    * nodeToken1 -> "SET"
    * columnValues -> ColumnValues()
    * nodeOptional1 -> [ "WHERE" ( SQLExpression() | "CURRENT" "OF" <S_IDENTIFIER> ) ]
    * nodeToken2 -> ";"
    */
   public void visit(UpdateStatement n) {
      n.nodeToken.accept(this);
      n.tableReference.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken1.accept(this);
      n.columnValues.accept(this);
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken2.accept(this);
   }

   /**
    * tableColumn -> TableColumn()
    * nodeToken -> "="
    * updatedValue -> UpdatedValue()
    * nodeListOptional -> ( "," TableColumn() "=" UpdatedValue() )*
    */
   public void visit(ColumnValues n) {
      n.tableColumn.accept(this);
      n.nodeToken.accept(this);
      n.updatedValue.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeChoice -> "(" SelectStatement() ")"
    *       | PlSqlExpression()
    */
   public void visit(UpdatedValue n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "INSERT"
    * nodeToken1 -> "INTO"
    * tableReference -> TableReference()
    * nodeOptional -> [ "(" TableColumn() ( "," TableColumn() )* ")" ]
    * nodeChoice -> ( "VALUES" "(" PlSqlExpressionList() ")" | SelectStatement() )
    * nodeToken2 -> ";"
    */
   public void visit(InsertStatement n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.tableReference.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeChoice.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeToken -> "DELETE"
    * nodeOptional -> [ "FROM" ]
    * tableReference -> TableReference()
    * nodeOptional1 -> [ <S_IDENTIFIER> ]
    * nodeOptional2 -> [ "WHERE" ( SQLExpression() | "CURRENT" "OF" <S_IDENTIFIER> ) ]
    * nodeToken1 -> ";"
    */
   public void visit(DeleteStatement n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.tableReference.accept(this);
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      if ( n.nodeOptional2.present() ) {
         n.nodeOptional2.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * nodeOptional -> [ CreateViewClause() ]
    * selectStatement -> SelectStatement()
    * nodeToken -> ";"
    */
   public void visit(QueryStatement n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.selectStatement.accept(this);
      n.nodeToken.accept(this);
   }

   /**
    * nodeToken -> "CREATE"
    * nodeToken1 -> "VIEW"
    * viewReference -> ViewReference()
    * createViewList -> CreateViewList()
    * nodeToken2 -> "AS"
    */
   public void visit(CreateViewClause n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.viewReference.accept(this);
      n.createViewList.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * plSqlAndExpression -> PlSqlAndExpression()
    * nodeListOptional -> ( "OR" PlSqlAndExpression() )*
    */
   public void visit(PlSqlExpression n) {
      n.plSqlAndExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * plSqlUnaryLogicalExpression -> PlSqlUnaryLogicalExpression()
    * nodeListOptional -> ( "AND" PlSqlUnaryLogicalExpression() )*
    */
   public void visit(PlSqlAndExpression n) {
      n.plSqlUnaryLogicalExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * plSqlRelationalExpression -> PlSqlRelationalExpression()
    */
   public void visit(PlSqlUnaryLogicalExpression n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.plSqlRelationalExpression.accept(this);
   }

   /**
    * plSqlSimpleExpression -> PlSqlSimpleExpression()
    * nodeOptional -> ( Relop() PlSqlSimpleExpression() | PlSqlInClause() | PlSqlBetweenClause() | PlSqlLikeClause() | IsNullClause() )?
    */
   public void visit(PlSqlRelationalExpression n) {
      n.plSqlSimpleExpression.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * plSqlExpression -> PlSqlExpression()
    * nodeListOptional -> ( "," PlSqlExpression() )*
    */
   public void visit(PlSqlExpressionList n) {
      n.plSqlExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "IN"
    * nodeToken1 -> "("
    * plSqlExpressionList -> PlSqlExpressionList()
    * nodeToken2 -> ")"
    */
   public void visit(PlSqlInClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.plSqlExpressionList.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "BETWEEN"
    * plSqlSimpleExpression -> PlSqlSimpleExpression()
    * nodeToken1 -> "AND"
    * plSqlSimpleExpression1 -> PlSqlSimpleExpression()
    */
   public void visit(PlSqlBetweenClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.plSqlSimpleExpression.accept(this);
      n.nodeToken1.accept(this);
      n.plSqlSimpleExpression1.accept(this);
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "LIKE"
    * plSqlSimpleExpression -> PlSqlSimpleExpression()
    */
   public void visit(PlSqlLikeClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.plSqlSimpleExpression.accept(this);
   }

   /**
    * nodeToken -> "IS"
    * nodeOptional -> [ "NOT" ]
    * nodeToken1 -> "NULL"
    */
   public void visit(IsNullClause n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * plSqlMultiplicativeExpression -> PlSqlMultiplicativeExpression()
    * nodeListOptional -> ( ( "+" | "-" | "||" ) PlSqlMultiplicativeExpression() )*
    */
   public void visit(PlSqlSimpleExpression n) {
      n.plSqlMultiplicativeExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * plSqlExpotentExpression -> PlSqlExpotentExpression()
    * nodeListOptional -> ( ( "*" | "/" ) PlSqlExpotentExpression() )*
    */
   public void visit(PlSqlMultiplicativeExpression n) {
      n.plSqlExpotentExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * plSqlUnaryExpression -> PlSqlUnaryExpression()
    * nodeListOptional -> ( "**" PlSqlUnaryExpression() )*
    */
   public void visit(PlSqlExpotentExpression n) {
      n.plSqlUnaryExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeChoice -> ( ( "+" | "-" ) PlSqlPrimaryExpression() )
    *       | PlSqlPrimaryExpression()
    */
   public void visit(PlSqlUnaryExpression n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeChoice -> "NULL"
    *       | ( <S_IDENTIFIER> ( "%FOUND" | "%NOTFOUND" | "%ISOPEN" | "%ROWCOUNT" ) )
    *       | ( <S_IDENTIFIER> "(" Arguments() ")" )
    *       | ( <S_IDENTIFIER> [ "." <S_IDENTIFIER> ] )
    *       | ( "SQL" ( "%FOUND" | "%NOTFOUND" | "%ISOPEN" | "%ROWCOUNT" ) )
    *       | <S_NUMBER>
    *       | <S_CHAR_LITERAL>
    *       | <S_BIND>
    *       | "(" PlSqlExpression() ")"
    */
   public void visit(PlSqlPrimaryExpression n) {
      n.nodeChoice.accept(this);
   }

   /**
    * oracleObjectName -> OracleObjectName()
    * nodeOptional -> [ "." OracleObjectName() [ "." OracleObjectName() ] ]
    */
   public void visit(TableColumn n) {
      n.oracleObjectName.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * nodeChoice -> <S_QUOTED_IDENTIFIER>
    *       | <S_IDENTIFIER>
    */
   public void visit(OracleObjectName n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeChoice -> "="
    *       | "!="
    *       | "#"
    *       | "<>"
    *       | ">"
    *       | ">="
    *       | "<"
    *       | "<="
    */
   public void visit(Relop n) {
      n.nodeChoice.accept(this);
   }

   /**
    * oracleObjectName -> OracleObjectName()
    * nodeOptional -> [ "." OracleObjectName() ]
    */
   public void visit(TableReference n) {
      n.oracleObjectName.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * oracleObjectName -> OracleObjectName()
    */
   public void visit(ViewReference n) {
      n.oracleObjectName.accept(this);
   }

   /**
    * parameter -> Parameter()
    * nodeListOptional -> ( "," Parameter() )*
    */
   public void visit(ParameterList n) {
      n.parameter.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeChoice -> <S_IDENTIFIER>
    *       | ( [ "+" | "-" ] <S_NUMBER> )
    */
   public void visit(NumOrID n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> <S_IDENTIFIER>
    * nodeOptional -> [ [ "IN" ] [ "OUT" ] TypeDeclaration() [ ( ":=" | "DEFAULT" ) PlSqlExpression() ] ]
    */
   public void visit(Parameter n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * plSqlExpressionList -> PlSqlExpressionList()
    */
   public void visit(Arguments n) {
      n.plSqlExpressionList.accept(this);
   }

   /**
    * selectWithoutOrder -> SelectWithoutOrder()
    * nodeOptional -> [ OrderByClause() ]
    * nodeOptional1 -> [ ForUpdateClause() ]
    */
   public void visit(SelectStatement n) {
      n.selectWithoutOrder.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
   }

   /**
    * nodeToken -> "SELECT"
    * nodeOptional -> [ "ALL" | "DISTINCT" ]
    * selectList -> SelectList()
    * nodeOptional1 -> [ IntoClause() ]
    * fromClause -> FromClause()
    * nodeOptional2 -> [ WhereClause() ]
    * nodeOptional3 -> [ ConnectClause() ]
    * nodeOptional4 -> [ GroupByClause() ]
    * nodeOptional5 -> [ SetClause() ]
    */
   public void visit(SelectWithoutOrder n) {
      n.nodeToken.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.selectList.accept(this);
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.fromClause.accept(this);
      if ( n.nodeOptional2.present() ) {
         n.nodeOptional2.accept(this);
      }
      if ( n.nodeOptional3.present() ) {
         n.nodeOptional3.accept(this);
      }
      if ( n.nodeOptional4.present() ) {
         n.nodeOptional4.accept(this);
      }
      if ( n.nodeOptional5.present() ) {
         n.nodeOptional5.accept(this);
      }
   }

   /**
    * nodeChoice -> "*"
    *       | SelectItem() ( "," SelectItem() )*
    */
   public void visit(SelectList n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeChoice -> OracleObjectName() ".*"
    *       | OracleObjectName() "." OracleObjectName() ".*"
    *       | SQLSimpleExpression() [ ColumnAlias() ]
    */
   public void visit(SelectItem n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> <S_IDENTIFIER>
    */
   public void visit(ColumnAlias n) {
      n.nodeToken.accept(this);
   }

   /**
    * nodeToken -> "("
    * createViewItem -> CreateViewItem()
    * nodeListOptional -> ( "," CreateViewItem() )*
    * nodeToken1 -> ")"
    */
   public void visit(CreateViewList n) {
      n.nodeToken.accept(this);
      n.createViewItem.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * oracleObjectName -> OracleObjectName()
    */
   public void visit(CreateViewItem n) {
      n.oracleObjectName.accept(this);
   }

   /**
    * nodeToken -> "INTO"
    * intoItem -> IntoItem()
    * nodeListOptional -> ( "," IntoItem() )*
    */
   public void visit(IntoClause n) {
      n.nodeToken.accept(this);
      n.intoItem.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeChoice -> ( <S_IDENTIFIER> [ "." <S_IDENTIFIER> ] )
    *       | ( <S_BIND> )
    */
   public void visit(IntoItem n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeToken -> "FROM"
    * schemaSQLFromItem -> SchemaSQLFromItem()
    * nodeListOptional -> ( "," SchemaSQLFromItem() )*
    */
   public void visit(FromClause n) {
      n.nodeToken.accept(this);
      n.schemaSQLFromItem.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeChoice -> SchemaSQL_RelationNames()
    *       | SchemaSQL_AttributeNames()
    *       | FromItem()
    */
   public void visit(SchemaSQLFromItem n) {
      n.nodeChoice.accept(this);
   }

   /**
    * tableReference -> TableReference()
    * nodeOptional -> [ TableAlias() ]
    */
   public void visit(FromItem n) {
      n.tableReference.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * nodeToken -> <O_ARROW>
    * tableAlias -> TableAlias()
    */
   public void visit(SchemaSQL_RelationNames n) {
      n.nodeToken.accept(this);
      n.tableAlias.accept(this);
   }

   /**
    * tableReference -> TableReference()
    * nodeToken -> <O_ARROW>
    * columnAlias -> ColumnAlias()
    */
   public void visit(SchemaSQL_AttributeNames n) {
      n.tableReference.accept(this);
      n.nodeToken.accept(this);
      n.columnAlias.accept(this);
   }

   /**
    * nodeToken -> <S_IDENTIFIER>
    */
   public void visit(TableAlias n) {
      n.nodeToken.accept(this);
   }

   /**
    * nodeToken -> "WHERE"
    * sQLExpression -> SQLExpression()
    */
   public void visit(WhereClause n) {
      n.nodeToken.accept(this);
      n.sQLExpression.accept(this);
   }

   /**
    * nodeSequence -> ( [ "START" "WITH" SQLExpression() ] "CONNECT" "BY" SQLExpression() [ "START" "WITH" SQLExpression() ] )
    */
   public void visit(ConnectClause n) {
      n.nodeSequence.accept(this);
   }

   /**
    * nodeToken -> "GROUP"
    * nodeToken1 -> "BY"
    * sQLExpressionList -> SQLExpressionList()
    * nodeOptional -> [ "HAVING" SQLExpression() ]
    */
   public void visit(GroupByClause n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.sQLExpressionList.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * nodeChoice -> ( ( "UNION" [ "ALL" ] ) | "INTERSECT" | "MINUS" )
    * nodeChoice1 -> ( ( "(" SelectWithoutOrder() ")" ) | SelectWithoutOrder() )
    */
   public void visit(SetClause n) {
      n.nodeChoice.accept(this);
      n.nodeChoice1.accept(this);
   }

   /**
    * nodeToken -> "ORDER"
    * nodeToken1 -> "BY"
    * sQLSimpleExpression -> SQLSimpleExpression()
    * nodeOptional -> [ "ASC" | "DESC" ]
    * nodeListOptional -> ( "," SQLSimpleExpression() [ "ASC" | "DESC" ] )*
    */
   public void visit(OrderByClause n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.sQLSimpleExpression.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeToken -> "FOR"
    * nodeToken1 -> "UPDATE"
    * nodeToken2 -> "OF"
    * tableColumn -> TableColumn()
    * nodeListOptional -> ( "," TableColumn() )*
    */
   public void visit(ForUpdateClause n) {
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
      n.tableColumn.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * sQLAndExpression -> SQLAndExpression()
    * nodeListOptional -> ( "OR" SQLAndExpression() )*
    */
   public void visit(SQLExpression n) {
      n.sQLAndExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * sQLUnaryLogicalExpression -> SQLUnaryLogicalExpression()
    * nodeListOptional -> ( "AND" SQLUnaryLogicalExpression() )*
    */
   public void visit(SQLAndExpression n) {
      n.sQLUnaryLogicalExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeChoice -> ExistsClause()
    *       | ( [ "NOT" ] SQLRelationalExpression() )
    */
   public void visit(SQLUnaryLogicalExpression n) {
      n.nodeChoice.accept(this);
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "EXISTS"
    * nodeToken1 -> "("
    * subQuery -> SubQuery()
    * nodeToken2 -> ")"
    */
   public void visit(ExistsClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.subQuery.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeChoice -> ( "(" SQLExpressionList() ")" | ( [ "PRIOR" ] SQLSimpleExpression() ) )
    * nodeOptional -> ( SQLRelationalOperatorExpression() | ( SQLInClause() ) | ( SQLBetweenClause() ) | ( SQLLikeClause() ) | IsNullClause() )?
    */
   public void visit(SQLRelationalExpression n) {
      n.nodeChoice.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
   }

   /**
    * sQLSimpleExpression -> SQLSimpleExpression()
    * nodeListOptional -> ( "," SQLSimpleExpression() )*
    */
   public void visit(SQLExpressionList n) {
      n.sQLSimpleExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * relop -> Relop()
    * nodeChoice -> ( ( [ "ALL" | "ANY" ] "(" SubQuery() ")" ) | [ "PRIOR" ] SQLSimpleExpression() )
    */
   public void visit(SQLRelationalOperatorExpression n) {
      n.relop.accept(this);
      n.nodeChoice.accept(this);
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "IN"
    * nodeToken1 -> "("
    * nodeChoice -> ( SQLExpressionList() | SubQuery() )
    * nodeToken2 -> ")"
    */
   public void visit(SQLInClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeChoice.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "BETWEEN"
    * sQLSimpleExpression -> SQLSimpleExpression()
    * nodeToken1 -> "AND"
    * sQLSimpleExpression1 -> SQLSimpleExpression()
    */
   public void visit(SQLBetweenClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.sQLSimpleExpression.accept(this);
      n.nodeToken1.accept(this);
      n.sQLSimpleExpression1.accept(this);
   }

   /**
    * nodeOptional -> [ "NOT" ]
    * nodeToken -> "LIKE"
    * sQLSimpleExpression -> SQLSimpleExpression()
    */
   public void visit(SQLLikeClause n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.sQLSimpleExpression.accept(this);
   }

   /**
    * sQLMultiplicativeExpression -> SQLMultiplicativeExpression()
    * nodeListOptional -> ( ( "+" | "-" | "||" ) SQLMultiplicativeExpression() )*
    */
   public void visit(SQLSimpleExpression n) {
      n.sQLMultiplicativeExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * sQLExpotentExpression -> SQLExpotentExpression()
    * nodeListOptional -> ( ( "*" | "/" ) SQLExpotentExpression() )*
    */
   public void visit(SQLMultiplicativeExpression n) {
      n.sQLExpotentExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * sQLUnaryExpression -> SQLUnaryExpression()
    * nodeListOptional -> ( "**" SQLUnaryExpression() )*
    */
   public void visit(SQLExpotentExpression n) {
      n.sQLUnaryExpression.accept(this);
      if ( n.nodeListOptional.present() ) {
         processList(n.nodeListOptional);
      }
   }

   /**
    * nodeOptional -> [ "+" | "-" ]
    * sQLPrimaryExpression -> SQLPrimaryExpression()
    */
   public void visit(SQLUnaryExpression n) {
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.sQLPrimaryExpression.accept(this);
   }

   /**
    * nodeChoice -> "NULL"
    *       | FunctionCall()
    *       | OuterJoinExpression()
    *       | TableColumn()
    *       | <S_NUMBER>
    *       | <S_CHAR_LITERAL>
    *       | <S_BIND>
    *       | "(" SQLExpression() ")"
    */
   public void visit(SQLPrimaryExpression n) {
      n.nodeChoice.accept(this);
   }

   /**
    * oracleObjectName -> OracleObjectName()
    * nodeOptional -> [ "." OracleObjectName() [ "." OracleObjectName() ] ]
    * nodeToken -> "("
    * nodeOptional1 -> [ [ "DISTINCT" | "ALL" ] ( SQLArguments() | "*" ) ]
    * nodeToken1 -> ")"
    */
   public void visit(FunctionCall n) {
      n.oracleObjectName.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      if ( n.nodeOptional1.present() ) {
         n.nodeOptional1.accept(this);
      }
      n.nodeToken1.accept(this);
   }

   /**
    * sQLExpressionList -> SQLExpressionList()
    */
   public void visit(SQLArguments n) {
      n.sQLExpressionList.accept(this);
   }

   /**
    * oracleObjectName -> OracleObjectName()
    * nodeOptional -> [ "." OracleObjectName() [ "." OracleObjectName() ] ]
    * nodeToken -> "("
    * nodeToken1 -> "+"
    * nodeToken2 -> ")"
    */
   public void visit(OuterJoinExpression n) {
      n.oracleObjectName.accept(this);
      if ( n.nodeOptional.present() ) {
         n.nodeOptional.accept(this);
      }
      n.nodeToken.accept(this);
      n.nodeToken1.accept(this);
      n.nodeToken2.accept(this);
   }

   /**
    * selectWithoutOrder -> SelectWithoutOrder()
    */
   public void visit(SubQuery n) {
      n.selectWithoutOrder.accept(this);
   }

}

class FormatCommand {
   public static final int FORCE = 0;
   public static final int INDENT = 1;
   public static final int OUTDENT = 2;
   public static final int SPACE = 3;

   private int command;
   private int numCommands;

   FormatCommand(int command, int numCommands) {
      this.command = command;
      this.numCommands = numCommands;
   }

   public int getCommand()             { return command; }
   public int getNumCommands()         { return numCommands; }
   public void setCommand(int i)       { command = i; }
   public void setNumCommands(int i)   { numCommands = i; }
}

class TreeFormatterException extends RuntimeException {
   TreeFormatterException()         { super(); }
   TreeFormatterException(String s) { super(s); }
}
